@@ -63,6 +63,7 @@ from ludwig.globals import (
     set_disable_progressbar,
     TRAIN_SET_METADATA_FILE_NAME,
 )
+from ludwig.models.calibrator import Calibrator
 from ludwig.models.ecd import ECD
 from ludwig.models.inference import InferenceModule
 from ludwig.models.predictor import (
@@ -86,7 +87,7 @@ from ludwig.utils.data_utils import (
 )
 from ludwig.utils.defaults import default_random_seed, merge_with_defaults
 from ludwig.utils.fs_utils import makedirs, open_file, path_exists, upload_output_directory
-from ludwig.utils.misc_utils import get_file_names, get_output_directory
+from ludwig.utils.misc_utils import get_file_names, get_output_directory, set_saved_weights_in_checkpoint_flag
 from ludwig.utils.print_utils import print_boxed
 from ludwig.utils.torch_utils import get_torch_device
 
@@ -420,11 +421,12 @@ class LudwigModel:
                         if key != "config":  # Config is printed separately.
                             experiment_description.append([key, pformat(value, indent=4)])
 
-                    print_boxed("EXPERIMENT DESCRIPTION")
-                    logger.info(tabulate(experiment_description, tablefmt="fancy_grid"))
+                    if self.backend.is_coordinator():
+                        print_boxed("EXPERIMENT DESCRIPTION")
+                        logger.info(tabulate(experiment_description, tablefmt="fancy_grid"))
 
-                    print_boxed("LUDWIG CONFIG")
-                    logger.info(pformat(self.config, indent=4))
+                        print_boxed("LUDWIG CONFIG")
+                        logger.info(pformat(self.config, indent=4))
 
                 for callback in self.callbacks:
                     callback.on_preprocess_start(self.config)
@@ -491,12 +493,14 @@ class LudwigModel:
                 update_config_with_metadata(self.config, training_set_metadata)
                 logger.info("Warnings and other logs:")
                 self.model = LudwigModel.create_model(self.config, random_seed=random_seed)
+                set_saved_weights_in_checkpoint_flag(self.config)
+
+            # Convert config dictionary into an instance of TrainerConfig.
+            trainer_config, _ = load_config_with_kwargs(Trainer.get_schema_cls(), self.config[TRAINER])
 
-            # init trainer
-            config, _ = load_config_with_kwargs(Trainer.get_schema_cls(), self.config[TRAINER])
             with self.backend.create_trainer(
                 model=self.model,
-                config=config,
+                config=trainer_config,
                 resume=model_resume_path is not None,
                 skip_save_model=skip_save_model,
                 skip_save_progress=skip_save_progress,
@@ -546,6 +550,27 @@ class LudwigModel:
                         save_path=model_dir,
                     )
 
+                    # Calibrates output feature probabilities on validation set if calibration is enabled.
+                    # Must be done after training, and before final model parameters are saved.
+                    if self.backend.is_coordinator():
+                        calibrator = Calibrator(
+                            trainer.model,
+                            self.backend,
+                            batch_size=trainer.eval_batch_size,
+                        )
+                        if validation_set is not None:
+                            # Use backend.createPredictor to ensure we get ray predictor with ray backend
+                            calibrator.train_calibration(validation_set, VALIDATION)
+                        else:
+                            logger.warning(
+                                "Calibration uses validation set, but not validation split specified. "
+                                "Will use training set for calibration."
+                            )
+                            calibrator.train_calibration(training_set, TRAINING)
+                        if not skip_save_model:
+                            model_weights_path = os.path.join(model_dir, MODEL_WEIGHTS_FILE_NAME)
+                            torch.save(self.model.state_dict(), model_weights_path)
+
                     # Unpack train()'s return.
                     # The statistics are all nested dictionaries of TrainerMetrics: feature_name -> metric_name ->
                     # List[TrainerMetric], with one entry per training checkpoint, according to steps_per_checkpoint.
@@ -615,10 +640,6 @@ class LudwigModel:
                     if not path_exists(weights_save_path):
                         with open_file(weights_save_path, "wb") as f:
                             torch.save(self.model.state_dict(), f)
-                    # Adds a flag to all input features indicating that the weights are saved in the checkpoint.
-                    for input_feature in self.config["input_features"]:
-                        input_feature["saved_weights_in_checkpoint"] = True
-                    self.save_config(model_dir)
 
                 # Synchronize model weights between workers
                 self.backend.sync_model(self.model)
@@ -681,6 +702,7 @@ class LudwigModel:
         if not self.model:
             update_config_with_metadata(self.config, training_set_metadata)
             self.model = LudwigModel.create_model(self.config, random_seed=random_seed)
+            set_saved_weights_in_checkpoint_flag(self.config)
 
         if not self._online_trainer:
             config, _ = load_config_with_kwargs(Trainer.get_schema_cls(), self.config[TRAINER])
@@ -1346,6 +1368,9 @@ class LudwigModel:
 
         config = backend.broadcast_return(lambda: load_json(os.path.join(model_dir, MODEL_HYPERPARAMETERS_FILE_NAME)))
 
+        # Upgrades deprecated fields and adds new required fields, in case the config loaded from disk is old.
+        config = merge_with_defaults(config)
+
         if backend_param is None and "backend" in config:
             # Reset backend from config
             backend = initialize_backend(config.get("backend"))
@@ -1362,6 +1387,7 @@ class LudwigModel:
         )
 
         # generate model from config
+        set_saved_weights_in_checkpoint_flag(config)
         ludwig_model.model = LudwigModel.create_model(config)
 
         # load model weights
