@@ -151,6 +151,7 @@ CONFIG = "config"
 COMBINER = "combiner"
 
 BALANCE_PERCENTAGE_TOLERANCE = 0.03
+IMBALANCE_DETECTION_RATIO = 0.05
 
 TABULAR = "tabular"
 AUTOML_DEFAULT_TABULAR_MODEL = "tabnet"
@@ -171,3 +172,5 @@ HYPEROPT_WARNING = (
 CONTINUE_PROMPT = "Do you want to continue? "
 
 DEFAULT_AUDIO_TENSOR_LENGTH = 70000
+
+DASK_MODULE_NAME = "dask.dataframe"
