@@ -21,7 +21,8 @@ import pandas as pd
 import torch
 
 from ludwig.backend import LOCAL_BACKEND
-from ludwig.utils.data_utils import DATAFRAME_FORMATS, DICT_FORMATS, to_numpy_dataset
+from ludwig.utils.data_utils import DATAFRAME_FORMATS, DICT_FORMATS
+from ludwig.utils.dataframe_utils import to_numpy_dataset
 from ludwig.utils.misc_utils import get_from_registry
 from ludwig.utils.strings_utils import make_safe_filename
 
@@ -40,7 +41,7 @@ def postprocess(
 
     saved_keys = set()
     if not skip_save_unprocessed_output:
-        _save_as_numpy(predictions, output_directory, saved_keys)
+        _save_as_numpy(predictions, output_directory, saved_keys, backend)
 
     for of_name, output_feature in output_features.items():
         predictions = output_feature.postprocess_predictions(
@@ -52,15 +53,15 @@ def postprocess(
 
     # Save any new columns but do not save the original columns again
     if not skip_save_unprocessed_output:
-        _save_as_numpy(predictions, output_directory, saved_keys)
+        _save_as_numpy(predictions, output_directory, saved_keys, backend)
 
     return predictions
 
 
-def _save_as_numpy(predictions, output_directory, saved_keys):
+def _save_as_numpy(predictions, output_directory, saved_keys, backend):
     predictions = predictions[[c for c in predictions.columns if c not in saved_keys]]
     npy_filename = os.path.join(output_directory, "{}.npy")
-    numpy_predictions = to_numpy_dataset(predictions)
+    numpy_predictions = to_numpy_dataset(predictions, backend)
     for k, v in numpy_predictions.items():
         k = k.replace("<", "[").replace(">", "]")  # Replace <UNK> and <PAD> with [UNK], [PAD]
         if k not in saved_keys:
