@@ -40,7 +40,7 @@ from ludwig.constants import (
     TYPE,
 )
 from ludwig.features.base_feature import BaseFeatureMixin, InputFeature, OutputFeature, PredictModule
-from ludwig.utils import output_feature_utils, strings_utils
+from ludwig.utils import calibration, output_feature_utils, strings_utils
 from ludwig.utils.eval_utils import (
     average_precision_score,
     ConfusionMatrix,
@@ -61,7 +61,7 @@ class _BinaryPreprocessing(torch.nn.Module):
         self.str2bool = str2bool or {v: True for v in strings_utils.BOOL_TRUE_STRS}
         self.should_lower = str2bool is None
 
-    def forward(self, v: TorchscriptPreprocessingInput):
+    def forward(self, v: TorchscriptPreprocessingInput) -> torch.Tensor:
         if torch.jit.isinstance(v, List[Tuple[torch.Tensor, int]]):
             raise ValueError(f"Unsupported input: {v}")
 
@@ -69,13 +69,13 @@ class _BinaryPreprocessing(torch.nn.Module):
             v = torch.stack(v)
 
         if torch.jit.isinstance(v, torch.Tensor):
-            return v.to(dtype=torch.bool)
+            return v.to(dtype=torch.float32)
 
         v = [s.strip() for s in v]
         if self.should_lower:
             v = [s.lower() for s in v]
         indices = [self.str2bool.get(s, False) for s in v]
-        return torch.tensor(indices, dtype=torch.bool)
+        return torch.tensor(indices, dtype=torch.float32)
 
 
 class _BinaryPostprocessing(torch.nn.Module):
@@ -102,13 +102,19 @@ class _BinaryPostprocessing(torch.nn.Module):
 
 
 class _BinaryPredict(PredictModule):
-    def __init__(self, threshold):
+    def __init__(self, threshold, calibration_module=None):
         super().__init__()
         self.threshold = threshold
+        self.calibration_module = calibration_module
 
     def forward(self, inputs: Dict[str, torch.Tensor], feature_name: str) -> Dict[str, torch.Tensor]:
         logits = output_feature_utils.get_output_feature_tensor(inputs, feature_name, self.logits_key)
-        probabilities = torch.sigmoid(logits)
+
+        if self.calibration_module is not None:
+            probabilities = self.calibration_module(logits)
+        else:
+            probabilities = torch.sigmoid(logits)
+
         predictions = probabilities >= self.threshold
         return {
             self.probabilities_key: probabilities,
@@ -149,8 +155,24 @@ class BinaryFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def cast_column(column, backend):
-        # todo maybe move code from add_feature_data here
-        #  + figure out what NaN is in a bool column
+        """Cast column of dtype object to bool.
+
+        Unchecked casting to boolean when given a column of dtype object converts all non-empty cells to True. We check
+        the values of the column directly and manually determine the best dtype to use.
+        """
+        values = backend.df_engine.compute(column.drop_duplicates())
+
+        if strings_utils.values_are_pandas_numbers(values):
+            # If numbers, convert to float so it can be converted to bool
+            column = column.astype(float).astype(bool)
+        elif strings_utils.values_are_pandas_bools(values):
+            # If booleans, manually assign boolean values
+            column = backend.df_engine.map_objects(
+                column, lambda x: x.lower() in strings_utils.PANDAS_TRUE_STRS
+            ).astype(bool)
+        else:
+            # If neither numbers or booleans, they are strings (objects)
+            column = column.astype(object)
         return column
 
     @staticmethod
@@ -287,8 +309,19 @@ class BinaryOutputFeature(BinaryFeatureMixin, OutputFeature):
             confidence_penalty=self.loss["confidence_penalty"],
         )
 
+    def create_calibration_module(self, feature) -> torch.nn.Module:
+        """Creates the appropriate calibration module based on the feature config.
+
+        Today, only one type of calibration ("temperature_scaling") is available, but more options may be supported in
+        the future.
+        """
+        if feature.get("calibration"):
+            calibration_cls = calibration.get_calibration_cls(BINARY, "temperature_scaling")
+            return calibration_cls(binary=True)
+        return None
+
     def create_predict_module(self) -> PredictModule:
-        return _BinaryPredict(self.threshold)
+        return _BinaryPredict(self.threshold, calibration_module=self.calibration_module)
 
     def get_prediction_set(self):
         return {PREDICTIONS, PROBABILITIES, LOGITS}
