@@ -140,15 +140,15 @@ class Trainer(BaseTrainer):
         :param resume: Resume training a model that was being trained. (default: False).
         :type resume: Boolean
         :param skip_save_model: Disables saving model weights and hyperparameters each time the model improves. By
-                default Ludwig saves model weights after each epoch the validation metric (improves, but if the model is
-                really big that can be time consuming. If you do not want to keep the weights and just find out what
-                performance a model can get with a set of hyperparameters, use this parameter to skip it, but the model
-                will not be loadable later on. (default: False).
+                default Ludwig saves model weights after each round of evaluation the validation metric (improves, but
+                if the model is really big that can be time consuming. If you do not want to keep the weights and just
+                find out what performance a model can get with a set of hyperparameters, use this parameter to skip it,
+                but the model will not be loadable later on. (default: False).
         :type skip_save_model: Boolean
-        :param skip_save_progress: Disables saving progress each epoch. By default Ludwig saves weights and stats after
-                each epoch for enabling resuming of training, but if the model is really big that can be time consuming
-                and will uses twice as much space, use this parameter to skip it, but training cannot be resumed later
-                on. (default: False).
+        :param skip_save_progress: Disables saving progress each round of evaluation. By default Ludwig saves weights
+                and stats after each round of evaluation for enabling resuming of training, but if the model is really
+                big that can be time consuming and will uses twice as much space, use this parameter to skip it, but
+                training cannot be resumed later on. (default: False).
         :type skip_save_progress: Boolean
         :param skip_save_log: Disables saving TensorBoard logs. By default Ludwig saves logs for the TensorBoard, but if
                 it is not needed turning it off can slightly increase the overall speed. (default: False).
@@ -355,11 +355,15 @@ class Trainer(BaseTrainer):
             while not batcher.last_batch() and step_count < total_steps:
                 batch = batcher.next_batch()
                 inputs = {
-                    i_feat.feature_name: torch.from_numpy(batch[i_feat.proc_column]).to(self.device)
+                    i_feat.feature_name: torch.from_numpy(np.array(batch[i_feat.proc_column], copy=True)).to(
+                        self.device
+                    )
                     for i_feat in self.model.input_features.values()
                 }
                 targets = {
-                    o_feat.feature_name: torch.from_numpy(batch[o_feat.proc_column]).to(self.device)
+                    o_feat.feature_name: torch.from_numpy(np.array(batch[o_feat.proc_column], copy=True)).to(
+                        self.device
+                    )
                     for o_feat in self.model.output_features.values()
                 }
 
@@ -421,11 +425,15 @@ class Trainer(BaseTrainer):
                 while not batcher.last_batch() and step_count < total_training_steps:
                     batch = batcher.next_batch()
                     inputs = {
-                        i_feat.feature_name: torch.from_numpy(batch[i_feat.proc_column]).to(self.device)
+                        i_feat.feature_name: torch.from_numpy(np.array(batch[i_feat.proc_column], copy=True)).to(
+                            self.device
+                        )
                         for i_feat in self.model.input_features.values()
                     }
                     targets = {
-                        o_feat.feature_name: torch.from_numpy(batch[o_feat.proc_column]).to(self.device)
+                        o_feat.feature_name: torch.from_numpy(np.array(batch[o_feat.proc_column], copy=True)).to(
+                            self.device
+                        )
                         for o_feat in self.model.output_features.values()
                     }
 
@@ -600,7 +608,7 @@ class Trainer(BaseTrainer):
             # eval metrics on validation set
             self.evaluation(
                 validation_set,
-                "vali",
+                VALIDATION,
                 progress_tracker.validation_metrics,
                 tables,
                 self.eval_batch_size,
@@ -747,9 +755,7 @@ class Trainer(BaseTrainer):
         checkpoint = checkpoint_manager = None
         if self.is_coordinator() and not self.skip_save_progress:
             checkpoint = Checkpoint(model=self.model, optimizer=self.optimizer)
-            checkpoint_manager = CheckpointManager(
-                checkpoint, training_checkpoints_path, device=self.device, max_to_keep=1
-            )
+            checkpoint_manager = CheckpointManager(checkpoint, training_checkpoints_path, device=self.device)
 
         train_summary_writer = None
         validation_summary_writer = None
@@ -973,11 +979,11 @@ class Trainer(BaseTrainer):
 
             # Move tensors to cuda here.
             inputs = {
-                i_feat.feature_name: torch.from_numpy(batch[i_feat.proc_column]).to(self.device)
+                i_feat.feature_name: torch.from_numpy(np.array(batch[i_feat.proc_column], copy=True)).to(self.device)
                 for i_feat in self.model.input_features.values()
             }
             targets = {
-                o_feat.feature_name: torch.from_numpy(batch[o_feat.proc_column]).to(self.device)
+                o_feat.feature_name: torch.from_numpy(np.array(batch[o_feat.proc_column], copy=True)).to(self.device)
                 for o_feat in self.model.output_features.values()
             }
 
@@ -1052,11 +1058,15 @@ class Trainer(BaseTrainer):
             while not batcher.last_batch():
                 batch = batcher.next_batch()
                 inputs = {
-                    i_feat.feature_name: torch.from_numpy(batch[i_feat.proc_column]).to(self.device)
+                    i_feat.feature_name: torch.from_numpy(np.array(batch[i_feat.proc_column], copy=True)).to(
+                        self.device
+                    )
                     for i_feat in self.model.input_features.values()
                 }
                 targets = {
-                    o_feat.feature_name: torch.from_numpy(batch[o_feat.proc_column]).to(self.device)
+                    o_feat.feature_name: torch.from_numpy(np.array(batch[o_feat.proc_column], copy=True)).to(
+                        self.device
+                    )
                     for o_feat in self.model.output_features.values()
                 }
 
@@ -1144,11 +1154,13 @@ class Trainer(BaseTrainer):
         # record how long its been since an improvement
         improved = get_improved_fun(validation_metric)
         validation_metrics = progress_tracker.validation_metrics[validation_output_feature_name]
-        if improved(validation_metrics[validation_metric][-1][-1], progress_tracker.best_eval_metric):
+        last_validation_metric = validation_metrics[validation_metric][-1]
+        last_validation_metric_value = last_validation_metric[-1]
+
+        if improved(last_validation_metric_value, progress_tracker.best_eval_metric):
             progress_tracker.last_improvement_steps = progress_tracker.steps
-            progress_tracker.best_eval_metric = progress_tracker.validation_metrics[validation_output_feature_name][
-                validation_metric
-            ][-1][-1]
+            progress_tracker.best_eval_metric = last_validation_metric_value
+
             if self.is_coordinator() and not skip_save_model:
                 torch.save(self.model.state_dict(), model_weights_path)
                 logger.info(
@@ -1182,10 +1194,10 @@ class Trainer(BaseTrainer):
                 and not progress_tracker.num_reductions_learning_rate >= reduce_learning_rate_on_plateau
             ):
                 logger.info(
-                    f"Last learning rate reduction happened {progress_tracker.last_learning_rate_reduction} epoch(s) "
+                    f"Last learning rate reduction happened {progress_tracker.last_learning_rate_reduction} step(s) "
                     f"ago, improvement of {validation_output_feature_name} {reduce_learning_rate_eval_split} "
                     f"{reduce_learning_rate_eval_metric} happened "
-                    f"{progress_tracker.last_reduce_learning_rate_eval_metric_improvement} epoch(s) ago."
+                    f"{progress_tracker.last_reduce_learning_rate_eval_metric_improvement} step(s) ago."
                 )
 
         # ========== Increase Batch Size Plateau logic =========
@@ -1211,10 +1223,10 @@ class Trainer(BaseTrainer):
             ):
                 logger.info(
                     "Last batch size increase "
-                    f"happened {progress_tracker.last_increase_batch_size} epoch(s) ago, "
+                    f"happened {progress_tracker.last_increase_batch_size} step(s) ago, "
                     f"improvement of {validation_output_feature_name} {increase_batch_size_eval_split} "
                     f"{increase_batch_size_eval_metric} happened "
-                    f"{progress_tracker.last_increase_batch_size_eval_metric_improvement} epoch(s) ago."
+                    f"{progress_tracker.last_increase_batch_size_eval_metric_improvement} step(s) ago."
                 )
 
         # ========== Early Stop logic ==========
@@ -1271,14 +1283,15 @@ class Trainer(BaseTrainer):
 
     def reduce_learning_rate(
         self,
-        progress_tracker,
-        validation_output_feature_name,
-        reduce_learning_rate_on_plateau,
-        reduce_learning_rate_on_plateau_patience,
-        reduce_learning_rate_on_plateau_rate,
-        reduce_learning_rate_eval_metric=LOSS,
-        reduce_learning_rate_eval_split=TRAINING,
+        progress_tracker: ProgressTracker,
+        validation_output_feature_name: str,
+        reduce_learning_rate_on_plateau: int,
+        reduce_learning_rate_on_plateau_patience: int,
+        reduce_learning_rate_on_plateau_rate: float,
+        reduce_learning_rate_eval_metric: str = LOSS,
+        reduce_learning_rate_eval_split: str = TRAINING,
     ):
+        """Uses the progress tracker to determine if the learning rate should be reduced."""
         if not (progress_tracker.num_reductions_learning_rate >= reduce_learning_rate_on_plateau):
 
             if reduce_learning_rate_eval_split == TRAINING:
@@ -1289,21 +1302,22 @@ class Trainer(BaseTrainer):
                 split_metrics = progress_tracker.test_metrics
 
             validation_metric = reduce_learning_rate_eval_metric
-            last_metric_value = split_metrics[validation_output_feature_name][validation_metric][-1]
+            last_metric: TrainerMetric = split_metrics[validation_output_feature_name][validation_metric][-1]
+            last_metric_value = last_metric[-1]
 
             improved = get_improved_fun(validation_metric)
             is_improved = improved(last_metric_value, progress_tracker.best_reduce_learning_rate_eval_metric)
             if is_improved:
                 # we update the best metric value and set it to the current one
-                # and reset last improvement epoch count
+                # and reset last improvement step count
                 progress_tracker.best_reduce_learning_rate_eval_metric = last_metric_value
                 progress_tracker.last_reduce_learning_rate_eval_metric_improvement = 0
             else:
                 progress_tracker.last_reduce_learning_rate_eval_metric_improvement += 1
                 if not is_improved and (
-                    # learning rate reduction happened more than N epochs ago
+                    # learning rate reduction happened more than N steps ago
                     progress_tracker.last_learning_rate_reduction >= reduce_learning_rate_on_plateau_patience
-                    # No improvement of the evaluation metric since more than N epochs ago
+                    # No improvement of the evaluation metric since more than N steps ago
                     and progress_tracker.last_reduce_learning_rate_eval_metric_improvement
                     >= reduce_learning_rate_on_plateau_patience
                 ):
@@ -1329,15 +1343,16 @@ class Trainer(BaseTrainer):
 
     def increase_batch_size(
         self,
-        progress_tracker,
-        validation_output_feature_name,
-        increase_batch_size_on_plateau,
-        increase_batch_size_on_plateau_patience,
-        increase_batch_size_on_plateau_rate,
-        increase_batch_size_on_plateau_max,
-        increase_batch_size_eval_metric=LOSS,
-        increase_batch_size_eval_split=TRAINING,
+        progress_tracker: ProgressTracker,
+        validation_output_feature_name: str,
+        increase_batch_size_on_plateau: int,
+        increase_batch_size_on_plateau_patience: int,
+        increase_batch_size_on_plateau_rate: float,
+        increase_batch_size_on_plateau_max: int,
+        increase_batch_size_eval_metric: str = LOSS,
+        increase_batch_size_eval_split: str = TRAINING,
     ):
+        """Uses the progress tracker to determine if the batch size should be increased."""
         if (
             not progress_tracker.num_increases_batch_size >= increase_batch_size_on_plateau
             and not progress_tracker.batch_size == increase_batch_size_on_plateau_max
@@ -1351,22 +1366,23 @@ class Trainer(BaseTrainer):
                 split_metrics = progress_tracker.test_metrics
 
             validation_metric = increase_batch_size_eval_metric
-            last_metric_value = split_metrics[validation_output_feature_name][validation_metric][-1]
+            last_metric = split_metrics[validation_output_feature_name][validation_metric][-1]
+            last_metric_value = last_metric[-1]
 
             improved = get_improved_fun(validation_metric)
             is_improved = improved(last_metric_value, progress_tracker.best_increase_batch_size_eval_metric)
             if is_improved:
                 # We update the best metric value and set it to the current one, and reset last
-                # improvement epoch count
+                # improvement step count
                 progress_tracker.best_increase_batch_size_eval_metric = last_metric_value
                 progress_tracker.last_increase_batch_size_eval_metric_improvement = 0
             else:
                 progress_tracker.last_increase_batch_size_eval_metric_improvement += 1
                 if not is_improved and (
-                    # Batch size increase happened more than N epochs ago
+                    # Batch size increase happened more than N steps ago
                     progress_tracker.last_increase_batch_size >= increase_batch_size_on_plateau_patience
                     and (
-                        # No improvement of the evaluation metric since more than N epochs ago
+                        # No improvement of the evaluation metric since more than N steps ago
                         progress_tracker.last_increase_batch_size_eval_metric_improvement
                         >= increase_batch_size_on_plateau_patience
                     )
