@@ -4,7 +4,7 @@ import torch
 import torch.nn as nn
 
 from ludwig.modules.normalization_modules import GhostBatchNormalization
-from ludwig.utils.entmax import entmax15, entmax_bisect, sparsemax
+from ludwig.utils.entmax import Entmax15, EntmaxBisect, Sparsemax
 from ludwig.utils.torch_utils import LudwigModule
 
 
@@ -127,9 +127,10 @@ class TabNet(LudwigModule):
 
             # entropy is used to penalize the amount of sparsity
             # in feature selection
-            total_entropy += (
-                torch.mean(torch.sum(-mask_values * torch.log(mask_values + 0.00001), dim=1)) / self.num_steps
-            )
+            if self.sparsity.item() != 0.0:
+                total_entropy += (
+                    torch.mean(torch.sum(-mask_values * torch.log(mask_values + 0.00001), dim=1)) / self.num_steps
+                )
 
             masks.append(torch.unsqueeze(torch.unsqueeze(mask_values, 0), 3))  # [1, b_s, i_s, 1]
 
@@ -182,10 +183,11 @@ class FeatureBlock(LudwigModule):
         self.size = size
         units = size * 2 if apply_glu else size
 
-        if shared_fc_layer:
+        # Initialize fc_layer before assigning to shared layer for torchscript compatibilty
+        self.fc_layer = nn.Linear(input_size, units, bias=False)
+        if shared_fc_layer is not None:
+            assert shared_fc_layer.weight.shape == self.fc_layer.weight.shape
             self.fc_layer = shared_fc_layer
-        else:
-            self.fc_layer = nn.Linear(input_size, units, bias=False)
 
         self.batch_norm = GhostBatchNormalization(
             units, virtual_batch_size=bn_virtual_bs, momentum=bn_momentum, epsilon=bn_epsilon
@@ -230,6 +232,13 @@ class AttentiveTransformer(LudwigModule):
         else:
             self.trainable_alpha = entmax_alpha
 
+        if self.entmax_mode == "sparsemax":
+            self.entmax_module = Sparsemax()
+        elif self.entmax_mode == "entmax15":
+            self.entmax_module = Entmax15()
+        else:
+            self.entmax_module = EntmaxBisect(alpha=self.trainable_alpha)
+
         self.feature_block = FeatureBlock(
             input_size,
             size,
@@ -258,13 +267,7 @@ class AttentiveTransformer(LudwigModule):
         # from the z_cumsum. Substacting the mean will cause z_cumsum to be close
         # to zero.
         # hidden = hidden - tf.math.reduce_mean(hidden, axis=1)[:, tf.newaxis]
-
-        if self.entmax_mode == "sparsemax":
-            return sparsemax(hidden)
-        elif self.entmax_mode == "entmax15":
-            return entmax15(hidden)
-        else:
-            return entmax_bisect(hidden, self.trainable_alpha)
+        return self.entmax_module(hidden)
 
     @property
     def input_shape(self) -> torch.Size:
@@ -282,7 +285,7 @@ class FeatureTransformer(LudwigModule):
         self,
         input_size: int,
         size: int,
-        shared_fc_layers: List = [],
+        shared_fc_layers: Optional[List] = None,
         num_total_blocks: int = 4,
         num_shared_blocks: int = 2,
         bn_momentum: float = 0.1,
@@ -290,6 +293,8 @@ class FeatureTransformer(LudwigModule):
         bn_virtual_bs: int = None,
     ):
         super().__init__()
+        if shared_fc_layers is None:
+            shared_fc_layers = []
         self.input_size = input_size
         self.num_total_blocks = num_total_blocks
         self.num_shared_blocks = num_shared_blocks
@@ -304,17 +309,16 @@ class FeatureTransformer(LudwigModule):
         # build blocks
         self.blocks = nn.ModuleList()
         for n in range(num_total_blocks):
-            if shared_fc_layers and n < len(shared_fc_layers):
-                # add shared blocks
-                self.blocks.append(FeatureBlock(input_size, size, **kwargs, shared_fc_layer=shared_fc_layers[n]))
+            # Ensure the sizes fed into FeatureBlock are correct regardless of presence of shared_fc_layer
+            if n == 0:
+                in_features = input_size
             else:
-                # build new blocks
-                if n == 0:
-                    # first block
-                    self.blocks.append(FeatureBlock(input_size, size, **kwargs))
-                else:
-                    # subsequent blocks
-                    self.blocks.append(FeatureBlock(size, size, **kwargs))
+                in_features = size
+
+            if shared_fc_layers and n < len(shared_fc_layers):
+                self.blocks.append(FeatureBlock(in_features, size, **kwargs, shared_fc_layer=shared_fc_layers[n]))
+            else:
+                self.blocks.append(FeatureBlock(in_features, size, **kwargs))
 
     def forward(self, inputs: torch.Tensor) -> torch.Tensor:
         # shape notation
