@@ -30,6 +30,9 @@ from ludwig.utils.misc_utils import get_from_registry
 from ludwig.utils.tokenizers import tokenizer_registry
 from ludwig.utils.types import Series
 
+PANDAS_TRUE_STRS = {"true"}
+PANDAS_FALSE_STRS = {"false"}
+
 BOOL_TRUE_STRS = {"yes", "y", "true", "t", "1", "1.0"}
 BOOL_FALSE_STRS = {"no", "n", "false", "f", "0", "0.0"}
 
@@ -89,6 +92,22 @@ def str2bool(v: str, fallback_true_label=None) -> bool:
     return v == fallback_true_label
 
 
+def values_are_pandas_numbers(values: List[str]):
+    """Returns True if values would be read by pandas as dtype float or int."""
+    for v in values:
+        try:
+            float(v)
+        except ValueError:
+            return False
+    return True
+
+
+def values_are_pandas_bools(values: List[str]):
+    """Returns True if values would be read by pandas as dtype bool."""
+    lowercase_values_set = {str(v).lower() for v in values}
+    return lowercase_values_set.issubset(PANDAS_FALSE_STRS | PANDAS_TRUE_STRS)
+
+
 def are_conventional_bools(values: List[Union[str, bool]]) -> bool:
     """Returns whether all values are conventional booleans."""
     for value in values:
