@@ -6,13 +6,18 @@ import tempfile
 import numpy as np
 import pandas as pd
 import pytest
-import ray
 
 from ludwig.api import LudwigModel
 from ludwig.backend import LocalBackend
-from ludwig.backend.ray import RayBackend
 from tests.integration_tests.utils import create_data_set_to_use, spawn
 
+try:
+    import ray
+
+    from ludwig.backend.ray import RayBackend
+except ImportError:
+    ray = None
+
 rs = np.random.RandomState(42)
 RAY_BACKEND_CONFIG = {
     "type": "ray",
@@ -169,9 +174,9 @@ def test_imbalance_ray(balance):
 def test_imbalance_local(balance):
     config = {
         "input_features": [
-            {"name": "Index", "column": "Index", "type": "numerical"},
-            {"name": "random_1", "column": "random_1", "type": "numerical"},
-            {"name": "random_2", "column": "random_2", "type": "numerical"},
+            {"name": "Index", "column": "Index", "type": "number"},
+            {"name": "random_1", "column": "random_1", "type": "number"},
+            {"name": "random_2", "column": "random_2", "type": "category"},
         ],
         "output_features": [{"name": "Label", "column": "Label", "type": "binary"}],
         "trainer": {"epochs": 2, "batch_size": 8},
