@@ -12,22 +12,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-import contextlib
 import os
 import tempfile
-from distutils.version import LooseVersion
 
 import numpy as np
 import pandas as pd
 import pytest
-import ray
 import torch
 
 from ludwig.api import LudwigModel
 from ludwig.backend import create_ray_backend, initialize_backend, LOCAL_BACKEND
-from ludwig.backend.ray import get_trainer_kwargs, RayBackend
 from ludwig.constants import BACKFILL, BALANCE_PERCENTAGE_TOLERANCE, COLUMN, NAME, TRAINER
-from ludwig.data.dataframe.dask import DaskEngine
 from ludwig.data.preprocessing import balance_data
 from ludwig.utils.data_utils import read_parquet
 from tests.integration_tests.utils import (
@@ -41,6 +36,8 @@ from tests.integration_tests.utils import (
     h3_feature,
     image_feature,
     number_feature,
+    RAY_BACKEND_CONFIG,
+    ray_start,
     sequence_feature,
     set_feature,
     text_feature,
@@ -49,34 +46,25 @@ from tests.integration_tests.utils import (
     vector_feature,
 )
 
-RAY_BACKEND_CONFIG = {
-    "type": "ray",
-    "processor": {
-        "parallelism": 2,
-    },
-    "trainer": {
-        "use_gpu": False,
-        "num_workers": 2,
-        "resources_per_worker": {
-            "CPU": 0.1,
-            "GPU": 0,
-        },
-    },
-}
+try:
+    import ray
 
+    from ludwig.backend.ray import get_trainer_kwargs, RayBackend
+    from ludwig.data.dataframe.dask import DaskEngine
 
-@contextlib.contextmanager
-def ray_start(num_cpus=2, num_gpus=None):
-    res = ray.init(
-        num_cpus=num_cpus,
-        num_gpus=num_gpus,
-        include_dashboard=False,
-        object_store_memory=150 * 1024 * 1024,
-    )
-    try:
-        yield res
-    finally:
-        ray.shutdown()
+    @ray.remote(num_cpus=1, num_gpus=1)
+    def train_gpu(config, dataset, output_directory):
+        model = LudwigModel(config, backend="local")
+        _, _, output_dir = model.train(dataset, output_directory=output_directory)
+        return os.path.join(output_dir, "model")
+
+    @ray.remote(num_cpus=1, num_gpus=0)
+    def predict_cpu(model_dir, dataset):
+        model = LudwigModel.load(model_dir, backend="local")
+        model.predict(dataset)
+
+except ImportError:
+    ray = None
 
 
 def run_api_experiment(config, dataset, backend_config, skip_save_processed_input=True):
@@ -319,11 +307,12 @@ def test_ray_audio(dataset_type, feature_type):
         audio_dest_folder = os.path.join(tmpdir, "generated_audio")
         input_features = [audio_feature(folder=audio_dest_folder, preprocessing=preprocessing_params)]
         output_features = [binary_feature()]
-        run_test_with_features(input_features, output_features, dataset_type=dataset_type)
+        run_test_with_features(input_features, output_features, dataset_type=dataset_type, nan_percent=0.1)
 
 
+@pytest.mark.parametrize("dataset_type", ["csv", "parquet"])
 @pytest.mark.distributed
-def test_ray_image():
+def test_ray_image(dataset_type):
     with tempfile.TemporaryDirectory() as tmpdir:
         image_dest_folder = os.path.join(tmpdir, "generated_images")
         input_features = [
@@ -336,7 +325,7 @@ def test_ray_image():
             ),
         ]
         output_features = [binary_feature()]
-        run_test_with_features(input_features, output_features)
+        run_test_with_features(input_features, output_features, dataset_type=dataset_type, nan_percent=0.1)
 
 
 @pytest.mark.skip(reason="flaky: ray is running out of resources")
@@ -462,20 +451,6 @@ def _run_train_gpu_load_cpu(config, data_parquet):
         ray.get(predict_cpu.remote(model_dir, data_parquet))
 
 
-@ray.remote(num_cpus=1, num_gpus=1)
-def train_gpu(config, dataset, output_directory):
-    model = LudwigModel(config, backend="local")
-    _, _, output_dir = model.train(dataset, output_directory=output_directory)
-    return os.path.join(output_dir, "model")
-
-
-@ray.remote(num_cpus=1, num_gpus=0)
-def predict_cpu(model_dir, dataset):
-    model = LudwigModel.load(model_dir, backend="local")
-    model.predict(dataset)
-
-
-@pytest.mark.skipif(LooseVersion(ray.__version__) < LooseVersion("1.12"), reason="Serialization issue before Ray 1.12")
 @pytest.mark.distributed
 def test_tune_batch_size_lr():
     with ray_start(num_cpus=2, num_gpus=None):
@@ -517,3 +492,21 @@ def test_ray_progress_bar():
         output_features,
         df_engine="dask",
     )
+
+
+@pytest.mark.parametrize("calibration", [True, False])
+@pytest.mark.distributed
+def test_ray_calibration(calibration):
+    input_features = [
+        number_feature(normalization="zscore"),
+        set_feature(),
+        binary_feature(),
+    ]
+    output_features = [
+        binary_feature(calibration=calibration),
+        category_feature(vocab_size=3, calibration=calibration),
+    ]
+    run_test_with_features(
+        input_features,
+        output_features,
+    )
